"""Plots read counts of gene expression data."""

import logging
from pathlib import Path
from typing import Union
from datetime import datetime

import pandas
import matplotlib.pyplot as plt


def plot_read_counts(
    dir_path: Union[str, Path],
    gene_id: str
) -> str:
    """Plots histogram of gene ID and read counts from .csv files.

    This function reads .csv files from a directory formatted
    "GeneID, Counts" and plots a histogram of read counts for a
    chosen geneID.

    Args:
        dir_path: Path of directory containing .csv files of read counts.
        gene_id: Name of gene to plot.

    Returns:
        String representing path to .svg plot file.

    Raises:
        ValueError: Path passed in does not exist.
        TypeError: Gene ID parameter passed is not a String.
        FileNotFoundError: File path does not contain any .csv files.
        KeyError: A file containing an entry of the chosen gene is missing
                  a count entry for it.
        TypeError: A file containing an entry of the chosen gene has a count
                   that is not a number.
    """
    LOGGER = logging.getLogger(__name__)

    dir_path = Path(dir_path)

    if not dir_path.exists():
        raise ValueError("Invalid Path")

    if not isinstance(gene_id, str):
        raise TypeError("Invalid gene ID")

    # identify csv files from directory, formatted ['Gene', 'Count']
    # if this is not immediately cast as list, on some python versions
    # it sets file_paths to null after a single call
    file_paths = list(dir_path.glob("*.csv"))
    file_paths.sort()

    if len(file_paths) == 0:
        raise FileNotFoundError("No .csv files found in chosen directory")

    gene_count_dict = {}
    file_counter = 0

    LOGGER.info("Running with %s and gene %s" % (dir_path, gene_id))

    for file_path in file_paths:
        # read in csv here via pandas
        df_read_counts = pandas.read_csv(str(file_path),
                                         header=None, names=['Gene', 'Count'])

        # confirm that gene entry exists for edge case where count may be deleted
        confirm_gene_presence = df_read_counts.loc[df_read_counts.Gene == gene_id,
                                                   'Gene'].tolist()

        # find gene_id within dataframe
        gene_count = df_read_counts.loc[df_read_counts.Gene == gene_id,
                                        'Count'].tolist()

        # # If an entry exists for the gene, but the count is missing, a KeyError is thrown
        if confirm_gene_presence and not gene_count:
            raise KeyError("%s.csv contains an entry for %s, but is "
                           "missing a count." % (file_path.stem, gene_id))

        # If an entry exists for the gene, but the count is not a number, a TypeError is thrown
        if confirm_gene_presence:
            try:
                for i, count in enumerate(gene_count):
                    gene_count[i] = int(count)

            except ValueError:
                raise TypeError("%s.csv contains an entry for %s, but is "
                                "not a number." % (file_path.stem, gene_id))

        if not gene_count:
            LOGGER.warning(f"File {file_path.stem}.csv does not contain an entry for {gene_id}, "
                           "this is interpreted as 0")
            gene_count = [0]

        if len(gene_count) > 1:
            LOGGER.warning(
                "%s.csv contains more than one entry for chosen gene - these will be summed"
                % (file_path.stem)
            )

        file_counter += 1
        # Uncomment to use file name as label -> does not autoscale pyplot
        # gene_count_dict[file_path.stem] = sum(gene_count)
        gene_count_dict[f"Read {file_counter}"] = sum(gene_count)

    LOGGER.info('Read complete from %s files' % (file_counter))

    # read in dictionary from read_counts and plot counts on the same histogram
    file_names = list(gene_count_dict.keys())
    counts = list(gene_count_dict.values())

    # create and show bar plot of read counts for chosen gene
    plt.bar(range(len(gene_count_dict)), counts, tick_label=file_names, width=0.4)
    plt.title('Counts for %s' % (gene_id))
    plt.autoscale(axis='both')
    plot_file_path = Path(f"{Path.cwd()}/plots/{get_file_name()}.svg")
    plt.savefig(plot_file_path)
    LOGGER.info(f"Plot successfully generated and saved at {plot_file_path}")

    return(plot_file_path)


def get_file_name() -> str:
    """Helper function to determine output filename."""
    datetime_str = datetime.now().strftime("%y%m%d")
    file_name_index = 1
    plot_file_name = f"reads_plot_{datetime_str}_{file_name_index}"
    # while(Path.exists(Path(f"{Path.cwd()}/plots/{plot_file_name}"))):
    #     file_name_index += 1
    # plot_file_name = f"reads_plot_{datetime_str}_{file_name_index}"
    return plot_file_name


def main():
    """Prompts user in cl to enter path and gene to plot."""
    # Initialize logger
    logging.basicConfig(
        format='[%(asctime)s: %(levelname)s] "%(module)s" %(message)s',
        level=logging.INFO)

    # Prompt user for input
    csv_path = input("\nPlease enter a path: ")
    gene_of_interest = input("\nPlease enter a gene name to plot: ")
    plot_read_counts(csv_path, gene_of_interest)


if __name__ == "__main__":
    main()
